# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# MenuSegment.py
# Python implementation of the Class MenuSegment
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:11:57 AM
# Original author: talve
#
#######################################################
from segments.Segment import Segment
from segments.MenuRecord import MenuRecord
from segments.SegmentFactory import SegmentFactory
from resourcedump_lib.utils import constants as cs
from resourcedump_lib.utils.Exceptions import DumpNotSupported

import struct
import sys


class MenuSegment(Segment):
    """This class is responsible for holding menu segment data.
    """

    RECORD_BITS_SIZE = 416
    RECORD_DWORDS_SIZE = 13

    _segment_type_id = cs.RESOURCE_DUMP_SEGMENT_TYPE_MENU

    menu_sub_header_struct = struct.Struct('HH')

    def __init__(self, data):
        """Initialize the class by setting the class data.
        """

        super().__init__(data)
        self.type = cs.RESOURCE_DUMP_SEGMENT_TYPE_MENU
        self.size, _ = self.unpack_segment_header()
        self.num_of_records = self.unpack_num_of_records()

        self.records = []
        for i in range(self.num_of_records):
            rec = MenuRecord(data[(8 + i * self.RECORD_DWORDS_SIZE * 4): (8 + (i + 1) * self.RECORD_DWORDS_SIZE * 4)])
            self.records.append(rec)

    def unpack_num_of_records(self):
        short_1, short_2 = self.menu_sub_header_struct.unpack_from(self.raw_data, 4)
        return short_2 if sys.byteorder == "big" else short_1

    def get_records(self):
        """get all the menu segment records.
        """
        return self.records

    def get_printable_records(self):
        """get all the menu segment records in a printable format.
        """
        recs = []
        for rec in self.records:
            recs.append(rec.convert_record_obj_to_printable_list())
        return recs

    def get_segment_type_by_segment_name(self, segment_type):
        """get the menu segment type by a given segment name.
           if not found, method will return the given segment type
        """
        for rec in self.records:
            if rec.segment_name == segment_type:
                segment_type = rec.segment_type
        return segment_type

    def verify_support(self, **kwargs):
        """check if the arguments matches with the menu data.
        """
        dump_type = kwargs["segment"]
        index1 = kwargs["index1"]
        index2 = kwargs["index2"]
        num_of_objs_1 = kwargs["numOfObj1"]
        num_of_objs_2 = kwargs["numOfObj2"]
        match_rec = None

        # Check whether dump type supported
        for rec in self.records:
            if rec.segment_type == dump_type or rec.segment_name == dump_type:
                match_rec = rec
                break

        if not match_rec:
            raise DumpNotSupported("Dump type: {0} is not supported".format(dump_type))

        # Check index1 attribute
        if not index1 and index1 != 0 and match_rec.must_have_index1:
            raise DumpNotSupported(
                "Dump type: {0} must have index1 attribute, and it wasn't provided".format(dump_type))

        if not index2 and index2 != 0 and match_rec.must_have_index2:
            raise DumpNotSupported(
                "Dump type: {0} must have index2 attribute, and it wasn't provided".format(dump_type))

        if index1 and not match_rec.supports_index1:
            raise DumpNotSupported(
                "Dump type: {0} does not support index1 attribute, and it was provided".format(dump_type))

        if index2 and not match_rec.supports_index2:
            raise DumpNotSupported(
                "Dump type: {0} does not support index2 attribute, and it was provided".format(dump_type))

        if not num_of_objs_1 and match_rec.must_have_num_of_obj1:
            raise DumpNotSupported(
                "Dump type: {0} must have numOfObj1 attribute, and it wasn't provided".format(dump_type))

        if not num_of_objs_2 and match_rec.must_have_num_of_obj2:
            raise DumpNotSupported(
                "Dump type: {0} must have numOfObj2 attribute, and it wasn't provided".format(dump_type))

        if num_of_objs_1 and not match_rec.supports_num_of_obj1:
            raise DumpNotSupported(
                "Dump type: {0} does not support numOfObj1 attribute, and it was provided".format(dump_type))

        if num_of_objs_2 and not match_rec.supports_num_of_obj2:
            raise DumpNotSupported(
                "Dump type: {0} does not support numOfObj2 attribute, and it was provided".format(dump_type))

        if num_of_objs_1 == cs.NUM_OF_OBJ_ALL and not match_rec.supports_all_num_of_obj1:
            raise DumpNotSupported(
                "Dump type: {0} does not support 'all' as numOfObj1 attribute".format(dump_type))

        if num_of_objs_2 == cs.NUM_OF_OBJ_ALL and not match_rec.supports_all_num_of_obj2:
            raise DumpNotSupported(
                "Dump type: {0} does not support 'all' as numOfObj2 attribute".format(dump_type))

        if num_of_objs_1 == cs.NUM_OF_OBJ_ACTIVE and not match_rec.supports_active_num_of_obj1:
            raise DumpNotSupported(
                "Dump type: {0} does not support 'active' as numOfObj1 attribute".format(dump_type))

        if num_of_objs_2 == cs.NUM_OF_OBJ_ACTIVE and not match_rec.supports_active_num_of_obj2:
            raise DumpNotSupported(
                "Dump type: {0} does not support 'active' as numOfObj2 attribute".format(dump_type))


SegmentFactory.register(cs.RESOURCE_DUMP_SEGMENT_TYPE_MENU, MenuSegment)



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# SegmentCreator.py
# Python implementation of the Class SegmentCreator
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:12:03 AM
# Original author: talve
#
#######################################################
from segments.SegmentFactory import SegmentFactory
from segments.ResourceSegment import ResourceSegment
from segments.Segment import unpack_segment_header
from resourcedump_lib.utils import constants as cs
from resourceparse_lib.utils.common_functions import is_resource_segment


class SegmentCreator:
    """this class is responsible for splitting  the raw data to segments and creating
    segments objects.
    """

    @classmethod
    def create(cls, raw_data, aggregate=False):
        """convert segments data into a segments objects by using SegmentFactory.
        """
        try:
            segments = []
            starting_resource_type = None

            current_index = 0
            end_index = len(raw_data)
            while current_index < end_index:
                seg_size_dword, seg_id = unpack_segment_header(raw_data, current_index)

                if seg_size_dword == 0:
                    raise Exception("raw_data didn't get smaller - found segment_size = 0")

                seg_size = seg_size_dword * cs.DWORD_SIZE
                raw_seg = raw_data[current_index:current_index + seg_size]
                seg_type = cls.get_seg_type_for_register_segments(seg_id)
                seg = SegmentFactory.create(seg_type, raw_seg)
                seg.resource_type = seg_id

                is_resource_segment = isinstance(seg, ResourceSegment)
                to_aggregate = aggregate and is_resource_segment and seg.unpack_aggregate_bit()

                if not to_aggregate:
                    if is_resource_segment:
                        starting_resource_type = seg_id
                    segments.append(seg)
                else:
                    if not starting_resource_type or seg_id != starting_resource_type:
                        raise Exception("Aggregate bit on first or control segment is illegal")
                    else:
                        segments[-1].aggregate(seg)

                current_index += seg_size

        except Exception as e:
            raise Exception("Failed to create segments with error: {0}".format(e))
        return segments

    @classmethod
    def get_seg_type_for_register_segments(cls, seg_type):
        """This method check if the segment type is a reference segment
           and return the right type of that segment.
        """
        if is_resource_segment(seg_type):
            return cs.RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE
        return seg_type



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


#######################################################
#
# resourceparse.py
# Python implementation of the Class ResourceParse
# Generated by Enterprise Architect
# Created on:      16-Dec-2019 14:11:58 AM
# Original author: talve
#
#######################################################
import tools_version
import sys
import os

if sys.version_info[0] < 3:
    print("Error: This tool supports python 3.x only. Exiting...")
    exit(1)

import argparse
from resourceparse_lib.utils import constants as cs
from resourceparse_lib.utils.common_functions import valid_path_arg_type
from resourceparse_lib.ResourceParseManager import ResourceParseManager
from resourceparse_lib.utils.Exceptions import ResourceParseException
from resourceparse_lib.parsers.ResourceParser import PARSER_CLASSES, parser_type

sys.path.append(os.path.join("common"))


class ResourceParse:
    """This class is responsible for the resource dump UI by handling the user inputs and
       and running the right command.
    """

    _arg_parser = None
    _common_usage = None
    _common_help = None

    @classmethod
    def _init_arg_parser(cls, prog=None):
        tool_name = prog if prog else os.path.basename(__file__).split('.')[0]

        description = \
            """Description:
    This tool parses and formats the output of "resourcedump" tool.
    There are several methods of parsing (see --parser).
""" if not prog else None

        cls._arg_parser = argparse.ArgumentParser(prog=tool_name, description=description, add_help=False)

        required_args = cls._arg_parser.add_argument_group('required arguments')
        optional_args = cls._arg_parser.add_argument_group('optional arguments')
        optional_args.add_argument("-p", "--parser", dest="resource_parser", type=parser_type, nargs="?", const=PARSER_CLASSES["adb"], default=PARSER_CLASSES["adb"] if not prog else PARSER_CLASSES["raw"], help="Available options: {}. Default: 'adb'. see (Parsing methods) ".format(list(PARSER_CLASSES.keys())))
        optional_args.add_argument("-o", "--out", help='Location of the output file')

        # Only args that added up until here will be included in the resourcedump help menu

        cls._common_usage = cls._arg_parser.format_usage().rstrip() + " [PARSE_METHOD_ARGUMENTS]\n"

        parse_methods_help = "Parse Methods:\n    The parse method can be chosen by the common --parser option. Below the description of each parse method and its arguments.\n\n"
        parse_methods_help += "\n".join(['Parse Method - "{}":\n'.format(parser_name) + parser_class.get_arg_parser(cls._arg_parser.prog).format_help() for parser_name, parser_class in PARSER_CLASSES.items()])

        cls._common_help = "\n".join(cls._arg_parser.format_help().split("\n")[1:]) + "\n" + parse_methods_help

        input_named = required_args.add_mutually_exclusive_group(required=True)
        input_named.add_argument("-d", "--dump-file", type=valid_path_arg_type, help='Location of the dump file used for parsing')
        input_named.add_argument("--segments_provided", action="store_true", help=argparse.SUPPRESS)

        optional_args.add_argument('--version', action='version', help="Shows the tool's version and exit", version=tools_version.GetVersionString(tool_name, None))
        optional_args.add_argument("-v", help='Verbosity notice', dest="verbose", default=0, action='count')
        optional_args.add_argument("-h", "--help", action="help", help="show this help message and exit")

        standalone_usage = cls._arg_parser.format_usage().rstrip() + " [PARSE_METHOD_ARGUMENTS]\n"
        standalone_help_body = "\n".join(cls._arg_parser.format_help().split("\n")[1:]) + "\n" + parse_methods_help
        standalone_help = standalone_usage + standalone_help_body

        cls._arg_parser.format_help = lambda: standalone_help

    @classmethod
    def get_help(cls, prog):
        if not cls._arg_parser:
            cls._init_arg_parser(prog)
        return cls._common_help, cls._common_usage

    @classmethod
    def run_arg_parse(cls, argv=None, prog=None):
        """This method run the arg parse and return the arguments from the UI.
        """
        if not cls._arg_parser:
            cls._init_arg_parser(prog)

        manager_args, remaining = cls._arg_parser.parse_known_args(argv)
        parsing_method_arg_parser = manager_args.resource_parser.get_arg_parser(cls._arg_parser.prog)

        parser_args = parsing_method_arg_parser.parse_args(remaining) if parsing_method_arg_parser else argparse.Namespace

        return manager_args, parser_args

    def run(self, argv=None, segments=None):
        """This method run the parser with the needed arguments
        """
        manager_args, parser_args = self.run_arg_parse(argv)
        creator = ResourceParseManager(manager_args, parser_args, segments)
        creator.parse()


if __name__ == '__main__':
    try:
        ResourceParse().run()
    except KeyboardInterrupt:
        print("Aborted by user.")
        sys.exit(1)
    except ResourceParseException as rpe:
        print("ResourceParse failed!\n{0}.\nExiting...".format(rpe))
        sys.exit(1)
    except Exception as e:
        print("FATAL ERROR!\n{0}.\nExiting...".format(e))
        sys.exit(1)



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# ResourceParser.py
# Created on:      12-Jun-2023 11:13:33 AM
# Original author: astrutsovsky
#
#######################################################

from resourceparse_lib.utils import constants as cs
from resourceparse_lib.utils.common_functions import calculate_aligned_offset, is_resource_segment, valid_path_arg_type
from resourceparse_lib.utils.Exceptions import ResourceParseException
from resourceparse_lib.parsers.ResourceParser import ResourceParser, PARSER_CLASSES
from resourceparse_lib.parsers.AdbParser import AdbParser
import os
import re
import math
import struct


class AdbResourceParser(ResourceParser):
    PARSER_TYPE = "adb"

    @classmethod
    def get_arg_parser(cls, prog=None):
        arg_parser = super().get_arg_parser(prog)

        required_named = arg_parser.add_argument_group('required arguments')
        required_named.add_argument("-a", "--adb-file", type=valid_path_arg_type, required=True, help='Location of the ADB file')

        optional_named = arg_parser.add_argument_group('optional arguments')
        optional_named.add_argument("-r", "--raw", action="store_true", help='Prints the raw data in addition to the parsed data')

        return arg_parser

    @staticmethod
    def get_description():
        return \
            """        This parse method receives an adb file and parses each segment data
        according to the adb layout of the node with segment_id attribute corresponds to the segment
"""

    def __init__(self, parser_args):
        self._adb_file_path = parser_args.adb_file
        self._adb_obj = AdbParser(self._adb_file_path)
        self._segment_map = self._adb_obj.segment_id_nodes_dict
        self._raw = parser_args.raw
        self._manager = parser_args.manager

    # Segment Parsing

    def get_title(self):
        return "Parse {0} segments:".format(self._manager.get_num_dumped_segments())

    def get_segment_separator(self):
        return cs.DATA_PRINTER_SEPARATOR

    def parse_segment(self, seg):
        """This method responsible for the parsing algorithm that take the raw data of
        each segment and generate his content according the adb map
        """

        seg_for_parse = False
        segment_parse_method = "adb"
        segment_layout = None

        if seg.get_type() in self._segment_map:
            seg_for_parse = True
            segment_layout = self._segment_map[seg.get_type()]
            segment_parse_method = segment_layout.parse_method

            segment_name = self._build_union_prefix(segment_layout.nodeDesc) \
                + segment_layout.name
            if segment_parse_method == "adb" and not self._is_seg_size_match_adb_seg_size(len(seg.get_data()), seg.get_type()):
                seg.add_parsed_data("Warning:{}".format(cs.WARNING_SIZE_DOESNT_MATCH.format(
                    (len(seg.get_data()) // cs.DWORD_SIZE) - cs.RESOURCE_SEGMENT_START_OFFSET_IN_DW,
                    math.ceil(segment_layout.size / 32))))
        else:
            # reference segment is missing in the adb (this will help the user understand the type)
            if seg.get_type() == cs.RESOURCE_DUMP_SEGMENT_TYPE_REFERENCE:
                segment_name = "segment_reference"
            else:
                segment_name = "UNKNOWN"

        seg.add_parsed_data(20 * " " + "Segment - {0} ({1:#06x}){2}".format(segment_name, seg.get_type(), seg.additional_title_info()))

        if is_resource_segment(seg.get_type()):
            data_start_position = cs.RESOURCE_SEGMENT_START_OFFSET_IN_BYTES
        else:
            data_start_position = 0

        # offset is in bits so converting is to bit array and store it at a member is better then pass it as an argument
        # for a recursive method
        bytes_array = seg.get_data('big')[data_start_position:]
        self._current_bit_array = ''.join(format(x, '08b') for x in bytes_array)

        if seg_for_parse:
            if segment_parse_method == 'adb':
                for field in self._get_union_selected_items(segment_layout):
                    prefix = self._build_union_prefix(field.nodeDesc)
                    self._parse_seg_field(field, prefix + field.name, 0, seg)
                if self._raw:
                    self._build_and_add_raw_data(seg)
            else:
                segment_resource_parser = PARSER_CLASSES.get(segment_parse_method)
                if segment_resource_parser:
                    segment_resource_parser([]).parse_segment(seg)
                else:
                    self._build_and_add_raw_data(seg)
        else:  # if segment not for parse, need to set raw data
            self._build_and_add_raw_data(seg)
        seg._parsed_data.extend(seg.get_messages())

    @classmethod
    def _build_and_add_raw_data(cls, seg):
        """This method build the raw data in the right format and add it to the
        parsed data of the segment.
        """
        hex_list = []
        line_counter = 0
        dw_counter = 0
        seg.add_parsed_data("RAW DATA:")

        for dw in struct.unpack("{}I".format(len(seg.get_data()) // cs.DWORD_SIZE), seg.get_data()):
            hex_list.append('0x{0:0{1}X} '.format(dw, 8))
            dw_counter += 1

            if (dw_counter % cs.PARSER_NUM_OF_DW_IN_ROW) == 0:
                seg.add_parsed_data("{:<15}:{}".format("DWORD [{0}-{1}]".format(line_counter * 4, (line_counter * 4) + (len(hex_list) - 1)), ''.join(hex_list[:])))
                line_counter += 1
                hex_list.clear()

        if len(hex_list) > 1:
            seg.add_parsed_data("{:<15}:{}".format("DWORD [{0}-{1}]".format(line_counter * 4, (line_counter * 4) + (len(hex_list) - 1)), ''.join(hex_list[:])))
        elif len(hex_list) == 1:
            seg.add_parsed_data("{:<15}:{}".format("DWORD [{0}]".format(line_counter * 4), ''.join(hex_list[:])))

    @classmethod
    def _build_union_prefix(cls, node_desc):
        """This method build the prefix for the struct in order to give the user
        information indicate if the node is a union
        """
        if not node_desc:
            prefix = ""
        else:
            if node_desc.isUnion:
                prefix = "(UNION)"
            else:
                prefix = ""
        return prefix

    def _is_seg_size_match_adb_seg_size(self, seg_size, seg_type):
        """This method check if the dumped segment size is same like in the ADB definitions.
        """
        adb_seg_size = math.ceil(self._segment_map[seg_type].size / 8)
        dumped_data_size = seg_size
        # first 4 dw's at the resource segment is not a part of the ADB definition so we not consider them
        if is_resource_segment(seg_type):
            dumped_data_size -= cs.RESOURCE_SEGMENT_START_OFFSET_IN_DW * cs.DWORD_SIZE
        if adb_seg_size != dumped_data_size:
            return False
        return True

    def _get_union_selected_items(self, field):
        """This method parse union field and present only the relevant field
        (selected by the selector)
        """
        if not field.uSelector:
            return field.subItems

        selector_field_size = field.uSelector.selector_layout_item.size
        selector_field_offset = calculate_aligned_offset(field.uSelector.selector_layout_item.offset, selector_field_size)
        selected_layout_item = field.uSelector.select_to_layout_item.get(int(self._current_bit_array[selector_field_offset:selector_field_offset + selector_field_size], 2))

        return [selected_layout_item] if selected_layout_item else field.subItems

    def _parse_seg_field(self, field, field_str, offset_shift, seg):  # offset_shift is 0 unless parsing a sub-tree of varaible-size arrays
        """This method is a recursive method that build the inner fields
        """
        try:
            if field.condition and not field.condition.evaluate(self._current_bit_array):
                return
        except ResourceParseException as rpe:
            seg.add_parsed_data("Warning:{}".format(cs.WARNING_FAILED_EVAL_CONDITION.format(str(rpe), field_str)))

        num_unlimited_elements = 1
        if field.is_unlimited_array:
            unlimited_array_start_offset = calculate_aligned_offset(field.offset, field.size)
            segment_remainder_size = len(self._current_bit_array) - unlimited_array_start_offset
            num_unlimited_elements = segment_remainder_size // field.size

        for i in range(num_unlimited_elements):
            array_suffix = "[" + str(i) + "]" if field.is_unlimited_array else ""
            element_field_str = field_str + array_suffix
            element_offset_shift = offset_shift + field.size * i

            if field.adb_enum:
                self._parse_enum_field(field, element_field_str, element_offset_shift, seg)

            elif len(field.subItems) > 0:
                for sub_field in self._get_union_selected_items(field):
                    prefix = self._build_union_prefix(sub_field.nodeDesc)
                    self._parse_seg_field(sub_field, element_field_str + "." + prefix + sub_field.name, element_offset_shift, seg)
            else:
                field_offset = calculate_aligned_offset(field.offset + element_offset_shift, field.size)
                if len(self._current_bit_array) >= (field_offset + field.size):
                    value = hex(int(self._current_bit_array[field_offset:field_offset + field.size], 2))
                    if ("printf" in field.attrs):
                        value = self._parse_printf_format(value, field.attrs["printf"])
                    seg.add_parsed_data("{} = {}".format(element_field_str, value))

    def _parse_enum_field(self, field, field_str, offset_shift, seg):
        """This method parse enum field and present the enum name as well as the enum value.
        """
        field_offset = calculate_aligned_offset(field.offset + offset_shift, field.size)
        enum_value = int(self._current_bit_array[field_offset:field_offset + field.size], 2)
        if len(self._current_bit_array) >= (field_offset + field.size):
            enum_string = field.adb_enum.num_to_string.get(enum_value)
            if enum_string is not None:
                seg.add_parsed_data("{} = ({} = {})".format(field_str, enum_string, hex(enum_value)))
            else:
                seg.add_parsed_data("{} = {}".format(field_str, hex(enum_value)))

    def _parse_printf_format(self, value, format):
        """ Enables specifying how a field should be dumped
            %d - dump field as decimal
            %x – dump field as hexadecimal
            %b - dump field as binary
            %s – dump field as ASCII tex"""

        cases = ['d', 'x', 'b', 's']
        new_val = ''
        tmp_val = value.split("0x")[1]
        str_list = []
        str_list[:0] = format

        for i, c in enumerate(str_list):
            f = ""
            if (i + 1 < len(str_list)):
                if (c == '%' or c + str_list[i + 1] == '0x' or c + str_list[i + 1] == c + 'x'):
                    if (c == '%' and str_list[i + 1] in cases):
                        f = c + str_list[i + 1]
                        del str_list[i]
                    if (i + 1 < len(str_list)):
                        if (c + str_list[i + 1] == '0x'):
                            new_val += '0x'
                            del str_list[i]
                    if (f == '%d'):
                        new_val += str(int(tmp_val, 16))
                    if (f == '%x'):
                        new_val += str(tmp_val)
                    if (f == '%b'):
                        new_val += str(bin(int(tmp_val, 16)).replace("0b", ""))
                    if (f == '%s'):
                        new_val += str(tmp_val)
                    if (i + 2 < len(str_list)):
                        if (c + str_list[i + 1] + str_list[i + 2] == '%' + str_list[i + 1] + 'x'):
                            new_val += (int(str_list[i + 1]) - 1) * ' ' + str(tmp_val)
                            for _ in range(2):
                                del str_list[i]
                    if (i + 2 < len(str_list)):
                        if (c + str_list[i + 1] + str_list[i + 2] == '%' + str_list[i + 1] + str_list[i + 2] and str_list[i + 1].isdigit()):
                            new_val += (int(str_list[i + 2]) - 1) * '0' + str(tmp_val)
                            for _ in range(3):
                                del str_list[i]
                else:
                    new_val += c
            else:
                new_val += c

        return new_val

    # Validation

    def validate(self):
        self._validate_adb_version_with_notice(self._adb_file_path)

    def _validate_adb_version_with_notice(self, adb_file_path):
        """This method perform the fw version validation stages and send a proper notice msg
        if needed.
        """
        adb_name = os.path.splitext(os.path.basename(adb_file_path))[0]
        if self._is_adb_name_in_version_format(adb_name):
            adb_fw_version = self._retrieve_adb_version_from_valid_format(adb_name)
            dump_fw_version = self._manager.retrieve_adb_version_from_info_segment()
            if adb_fw_version != dump_fw_version:
                # self._get_next_notice_counter() todo: return this mechanism
                self._manager._printer.print_notice_before_parse("Notice - {0}".format(
                    "adb fw version {0} is used for parsing while dump fw version is {1}".format(adb_fw_version,
                                                                                                 dump_fw_version)))
        else:
            # self._get_next_notice_counter() todo: return this mechanism
            self._manager._printer.print_notice_before_parse(
                "Notice - {0}".format("Adb name is not according the version format, can't validate adb fw version"))

    @classmethod
    def _is_adb_name_in_version_format(cls, adb_name):
        """This method check if the adb file name has the version format define by arch.
        """
        match = re.fullmatch("fw-[0-9]+-rel-[0-9]{2}_[0-9]{2}_[0-9]{4}", adb_name)
        if match is not None:
            if match.string == adb_name:
                return True
        return False

    @classmethod
    def _retrieve_adb_version_from_valid_format(cls, adb_name):
        """This method extract the fw_version from the file name assuming that
        file already checked and has the right version format.
        """
        version = re.findall("[0-9]{2}_[0-9]{2}_[0-9]{4}", adb_name)[0]
        version_list = version.split("_")
        return "{0}.{1}.{2}".format(version_list[0], version_list[1], version_list[2])

    def _get_next_warning_counter(self):  # todo: return this mechanism
        """This method return get the next free index to help generate
         the a uniq key for the warning msg.
        """
        return 5


PARSER_CLASSES[AdbResourceParser.PARSER_TYPE] = AdbResourceParser



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# ErrorSegment.py
# Python implementation of the Class ErrorSegment
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:11:57 AM
# Original author: talve
#
#######################################################
from segments.Segment import Segment
from segments.SegmentFactory import SegmentFactory
from resourceparse_lib.utils.common_functions import reverse_string_endian
from resourcedump_lib.utils import constants

import struct
import sys


class ErrorSegment(Segment):
    """this class is responsible for holding error segment data.
    """

    message_name = "Error message"
    _segment_type_id = constants.RESOURCE_DUMP_SEGMENT_TYPE_ERROR

    error_struct = struct.Struct('HHII32s')

    def __init__(self, data):
        """initialize the class by setting the class data.
        """
        super().__init__(data)

    def get_messages(self):
        start = self.segment_header_struct.size
        if len(self.raw_data) > start:
            syndrome_id, error_message = self.unpack_error()
            self.add_message("{}{} = {}".format(self.message_name, " ({})".format(syndrome_id) if syndrome_id else "", error_message))

        return self._messages

    def unpack_error(self):
        fields = self.error_struct.unpack_from(self.raw_data, self.segment_header_struct.size)
        if sys.byteorder == 'big':
            syndrome_id = fields[1]
            error_message = fields[4]
        else:
            syndrome_id = fields[0]
            error_message = reverse_string_endian(fields[4])
        # syndrome_id, error_message = fields[1], fields[4]
        error_message = error_message.decode('utf-8')
        return syndrome_id, error_message


SegmentFactory.register(constants.RESOURCE_DUMP_SEGMENT_TYPE_ERROR, ErrorSegment)



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# ResourceSegment.py
# Python implementation of the Class ResourceSegment
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:11:57 AM
# Original author: talve
#
#######################################################
from segments.Segment import Segment
from segments.SegmentFactory import SegmentFactory
from resourcedump_lib.utils import constants

import struct
import sys


class ResourceSegment(Segment):
    """this class is responsible for holding Resource segment data.
    """

    AGGREGATE_BIT_MASK = 0x00000001

    _segment_type_id = constants.RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE

    aggregate_dword_struct = struct.Struct('I')
    indices_struct = struct.Struct('II')

    def __init__(self, data):
        """initialize the class by setting the class data.
        """
        super().__init__(data)
        self.resource_type = constants.RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE
        self.index1, self.index2 = self.unpack_indices()  # TODO: for optimization move this line to later stage, before parsing

    def get_type(self):
        """get the general segment type.
        """
        return self.resource_type

    def unpack_aggregate_bit(self):
        aggregate_dword, = self.aggregate_dword_struct.unpack_from(self.raw_data, self.segment_header_struct.size)  # todo: test aggregate big/little
        return aggregate_dword & self.AGGREGATE_BIT_MASK

    def aggregate(self, other):
        data_start = self.segment_header_struct.size + self.aggregate_dword_struct.size + self.indices_struct.size
        self.raw_data += other.raw_data[data_start:]

    def unpack_indices(self):
        index1, index2 = self.indices_struct.unpack_from(self.raw_data, self.segment_header_struct.size + self.aggregate_dword_struct.size)
        return index1, index2

    def additional_title_info(self):
        """return index1 and index2 if exists in the segment.
        """
        return " ; index1 = {0}, index2 = {1}".format(hex(self.index1), hex(self.index2))


SegmentFactory.register(constants.RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE, ResourceSegment)



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# constants.py
# Python implementation of resource dump constants
# This file will supply all the constants needed
# for the resource dump tool
# Original author: romany
#
#######################################################

TOOL_NAME = "ResourceDump"
RESOURCE_DUMP_COMMAND_TYPE_QUERY = "query"
RESOURCE_DUMP_COMMAND_TYPE_DUMP = "dump"

RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE = 0xffffffff
RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE_MIN = 0x0000
RESOURCE_DUMP_SEGMENT_TYPE_RESOURCE_MAX = 0xfeff
RESOURCE_DUMP_SEGMENT_TYPE_NOTICE = 0xfff9
RESOURCE_DUMP_SEGMENT_TYPE_COMMAND = 0xfffa
RESOURCE_DUMP_SEGMENT_TYPE_TERMINATE = 0xfffb
RESOURCE_DUMP_SEGMENT_TYPE_ERROR = 0xfffc
RESOURCE_DUMP_SEGMENT_TYPE_INFO = 0xfffe
RESOURCE_DUMP_SEGMENT_TYPE_MENU = 0xffff
RESOURCE_DUMP_SEGMENT_TYPE_REFERENCE = 0xfffd


DEVICE_RDMA = "__AUTO__"
INF_DEPTH = -1 + 2**32
DEFAULT_VHCA = 0xFFFF

DWORD_SIZE = 4

# segment start/end specify where the 'type' attr starts and ends in the raw_data of each segment.
# should be 0-16 bits
SEGMENT_TYPE_DWORD_LOCATION = 0
SEGMENT_SIZE_DWORD_LOCATION = 0
SEGMENT_TYPE_START = 16
SEGMENT_TYPE_END = 32
SEGMENT_SIZE_START = 0
SEGMENT_SIZE_END = 16
MENU_SEGMENT_NUM_OF_RECORDS_DWORD_LOCATION = 1
MENU_SEGMENT_NUM_OF_RECORDS_START = 16
MENU_SEGMENT_NUM_OF_RECORDS_END = 32

MENU_SEGMENT_TYPE_DWORD_LOCATION = 0
MENU_SEGMENT_DWORD_SUPPORT_ATTRS_LOCATION = 0
MENU_SEGMENT_SEGMENT_NAME_DWORD_LOCATION_START = 1
MENU_SEGMENT_SEGMENT_NAME_DWORD_LOCATION_END = 5
MENU_SEGMENT_INDEX_NAME_1_DWORD_LOCATION_START = 5
MENU_SEGMENT_INDEX_NAME_1_DWORD_LOCATION_END = 9
MENU_SEGMENT_INDEX_NAME_2_DWORD_LOCATION_START = 9
MENU_SEGMENT_INDEX_NAME_2_DWORD_LOCATION_END = 13

NUM_OF_OBJ_ALL = 65535
NUM_OF_OBJ_ACTIVE = 65534

MENU_SEGMENT_TABLE_SEGMENT_TYPE = "Segment Type - "
MENU_SEGMENT_TABLE_DUMP_PARAMS = "Dump Params"
MENU_SEGMENT_TABLE_APPLICABILITY = "Applicability"
MENU_SEGMENT_TABLE_SPECIAL_VALUES = "Special Values"
MENU_SEGMENT_TABLE_MANDATORY = "Mandatory"
MENU_SEGMENT_TABLE_OPTIONAL = "Optional"
MENU_SEGMENT_TABLE_INDEX1 = "index1"
MENU_SEGMENT_TABLE_INDEX2 = "index2"
MENU_SEGMENT_TABLE_NUM_OF_OBJS1 = "num_of_obj1"
MENU_SEGMENT_TABLE_NUM_OF_OBJS2 = "num_of_obj2"
MENU_SEGMENT_TABLE_ALL = "all"
MENU_SEGMENT_TABLE_ACTIVE = "active"
MENU_SEGMENT_TABLE_LINE_LEN = 66
MENU_SEGMENT_TABLE_DUMP_PARAMS_LEN = 32
MENU_SEGMENT_TABLE_APPLICABILITY_LEN = 14
MENU_SEGMENT_TABLE_SPECIAL_VALUES_LEN = 14
MENU_SEGMENT_TABLE_GAP = 3
MENU_SEGMENT_TABLE_NA = "N/A"
MENU_SEGMENT_TABLE_DASH = "-"
MENU_SEGMENT_TABLE_AND = ","
MENU_SEGMENT_TABLE_WRAP = "_"



# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#######################################################
#
# DumpCommand.py
# Python implementation of the Class DumpCommand
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:12:00 AM
# Original author: talve
#
#######################################################
from resourcedump_lib.commands.ResDumpCommand import ResDumpCommand
from resourcedump_lib.commands.QueryCommand import QueryCommand
from resourcedump_lib.commands.CommandFactory import CommandFactory
from resourcedump_lib.utils import constants as cs
from resourcedump_lib.validation.CapabilityValidator import CapabilityValidator

from resourcedump_lib.cresourcedump.CResourceDump import CResourceDump
from resourcedump_lib.cresourcedump.cresourcedump_types import c_device_attributes, c_dump_request


class DumpCommand(ResDumpCommand):
    """This class is responsible for performing the dump command flow by validate,
    getting the data and print it.
    """
    def __init__(self, **kwargs):
        """DumpCommand initialization.
        """
        super().__init__()

        self.device_name = kwargs['device']
        self.segment = kwargs['segment']
        self.vHCAid = kwargs.get('vHCAid', cs.DEFAULT_VHCA)
        self.index1 = kwargs.get('index1', 0)
        self.index2 = kwargs.get('index2', 0)
        self.numOfObj1 = kwargs.get('numOfObj1', 0)
        self.numOfObj2 = kwargs.get('numOfObj2', 0)
        self.depth = kwargs.get('depth', cs.INF_DEPTH)
        self.bin = kwargs.get('bin', None)
        self.mem = kwargs.get('mem', "")

    def retrieve_data(self):
        """Validate request availability with QueryCommand and retrieve dump data using SDK.
        """
        menu = QueryCommand(device=self.device_name, vHCAid=self.vHCAid, mem=self.mem).get_segments()[0]

        # validate that the dump supported by calling ArgToMenuVerifier
        menu.verify_support(segment=self.segment, index1=self.index1, index2=self.index2, numOfObj1=self.numOfObj1, numOfObj2=self.numOfObj2)
        # segment type can be name, this method will convert the name (if needed) to seg number in hex (str)
        if (type(self.segment is str)):
            self.segment = menu.get_segment_type_by_segment_name(self.segment)

        if not self.bin:
            self.retrieve_data_from_sdk()
        else:
            self.dump_to_file_with_sdk()

    def dump_to_file_with_sdk(self):
        device_name = bytes(self.device_name, "utf-8")
        rdma_name = bytes(self.mem, "utf-8")

        device_attrs = c_device_attributes(device_name, self.vHCAid, rdma_name)
        dump_request = c_dump_request(self.segment, self.index1, self.index2, self.numOfObj1, self.numOfObj2)
        bin_filename = bytes(self.bin, "utf-8")
        # dump to file is done with BE as default as convention
        if CResourceDump.c_dump_resource_to_file(device_attrs, dump_request, self.depth, bin_filename, 1) != 0:
            raise Exception(CResourceDump.c_get_resource_dump_error().decode("utf-8"))
        print("write to file: ", self.bin)

    def validate(self):
        """call the capability validator and check if the core dump supported by the FW.
        """
        validation_status = False

        if CapabilityValidator.validate():
            validation_status = True
        else:
            print("resource dump register is not supported by FW")

        return validation_status


CommandFactory.register(cs.RESOURCE_DUMP_COMMAND_TYPE_DUMP, DumpCommand)



# Copyright (c) Sep 2019 Mellanox Technologies LTD. All rights reserved.
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --

import sys
try:
    from regaccess import RegAccException, extractField
except Exception as e:
    print("-E- could not import : %s" % str(e))
    sys.exit(1)


class CmdRegMrsi():

    ECOS_VALS = [
        {0, 'Reset/Boot-ROM'},
        {1, 'BL2'},
        {2, 'BL31'},
        {3, 'UEFI'},
        {4, 'OS starting'},
        {5, 'OS is running'},
        {6, 'Low - Power standby'},
        {7, 'Firmware update in progress'},
        {8, 'OS Crash Dump in progress'},
        {9, 'OS Crash Dump is complete'},
        {10, 'FW Fault Crash Dump in progress'},
        {11, 'FW Fault Crash Dump is complete'}
    ]

    LOW_POWER_STANDBY = 6
    EMBEDDED_CPU_DEVICE = 1

    def __init__(self, reg_access):
        self._reg_access = reg_access
        self._mrsi_get_result = None

    def get_mrsi(self, device=0):
        if self._mrsi_get_result is None:
            self._mrsi_get_result = self._reg_access.getMRSI(device)
        return self._mrsi_get_result  # returns {"reset_reason":value of reset_reason, "crts":value of crts, "ecos":value of ecos}

    def is_warm_reset(self):
        mrsi_get_result = self.get_mrsi()
        if mrsi_get_result["reset_reason"] == 1:
            return True
        return False

    def get_crts(self):
        mrsi_get_result = self.get_mrsi()
        return mrsi_get_result["crts"]

    def get_ecos(self, device=0):
        mrsi_get_result = self._reg_access.getMRSI(device)
        return mrsi_get_result["ecos"]

    def query_text(self, is_bluefield):
        try:
            warm_reset = self.is_warm_reset()
            timestamp = self.get_crts()
            reset_reason = "Warm reset" if warm_reset else "Cold reset"
        except Exception as e:
            reset_reason = "Unknown. Please check driver is up."
            timestamp = "Unknown. Please check driver is up."

        result = ""
        result += "{0}: {1:<51}{2:<14}\n".format("Reset-reason", "", reset_reason)
        result += "{0}: {1:<7}{2:<14}\n".format("Timestamp (number of clock cycles) since last cold reset", "", timestamp)

        # if is_bluefield:  # will be uncommented when FW will implement their support.
        #     ecos_result = self.ecos_query_text()
        #     result += ecos_result

        return result

    def ecos_query_text(self):
        state = self.get_ecos()
        description = CmdRegMrsi.ecos_vals[state]

        result = ""
        result = "Embedded CPU OS state:\n"
        result += "{0}: {1:<62}-{2:<14}{3}\n".format(state, description, "", "")
        return result
